import React from "react";
import { refnew } from "refnew";
import shallowequal from "shallowequal";
export function create() {
    /**
     * Map for calculates bitmask.
     */
    const selects = new Map();
    /**
     * React Context.
     */
    const { Provider, Consumer } = React.createContext({}, (_, next) => {
        const bits = Array.from(selects.entries()).reduce((bits, [select, { bit, prev }]) => {
            if (!shallowequal(select(next), prev)) {
                return bits | bit;
            }
            return bits;
        }, 0);
        return bits;
    });
    /**
     * RefnewProvider.
     */
    const RefnewProvider = class RefnewProvider extends React.Component {
        constructor(props) {
            super(props);
            this.state = {
                version: 0,
                value: refnew({
                    state: props.value
                })
            };
        }
        componentDidMount() {
            updator = (mutator) => {
                mutator && mutator(this.props.value);
                this.setState({
                    version: this.state.version + 1
                });
            };
        }
        componentWillUnmount() {
            updator = (_) => { };
        }
        render() {
            const { children } = this.props;
            return React.createElement(Provider, { value: this.state.value.state }, children);
        }
    };
    /**
     * RefnewConsumer.
     */
    const RefnewConsumer = class RefnewConsumer extends React.Component {
        constructor() {
            super(...arguments);
            this.state = { bit: 0 };
        }
        static getDerivedStateFromProps(props) {
            if (!selects.has(props.select)) {
                selects.set(props.select, { bit: 1 << selects.size });
            }
            return { bit: selects.get(props.select).bit };
        }
        shouldComponentUpdate() {
            return false;
        }
        render() {
            return (React.createElement(Consumer, { unstable_observedBits: this.state.bit }, state => this.select(state)));
        }
        select(state) {
            const { children, select } = this.props;
            return children((selects.get(select).prev = select(state)));
        }
    };
    let updator;
    return {
        Provider: RefnewProvider,
        Consumer: RefnewConsumer,
        update: (mutator) => {
            updator && updator(mutator);
        }
    };
}
//# sourceMappingURL=index.js.map